"""
Menu utilities.
"""
from fnmatch import fnmatch

from django.utils.importlib import import_module
from django.core.urlresolvers import reverse

from wpadmin.utils import (
    get_wpadmin_settings, get_admin_site, get_admin_site_name)


def get_menu_cls(menu, admin_site_name='admin'):
    """
    menu - menu name ('top' or 'left')
    """
    return get_wpadmin_settings(admin_site_name).get('menu', {}).get(menu, None)


def get_menu(menu, admin_site_name='admin'):
    """
    menu - menu name ('top' or 'left')
    """
    menu_cls = get_menu_cls(menu, admin_site_name)
    if menu_cls:
        mod, inst = menu_cls.rsplit('.', 1)
        mod = import_module(mod)
        return getattr(mod, inst)()
    return None


# I had to copy (and slightly modify) those utils from django-admin-tools
# to override get_admin_site
def get_avail_models(context):
    """ Returns (model, perm,) for all models user can possibly see """
    items = []
    admin_site = get_admin_site(context)

    for model, model_admin in list(admin_site._registry.items()):
        perms = model_admin.get_model_perms(context.get('request'))
        if True not in list(perms.values()):
            continue
        items.append((model, perms,))
    return items


def filter_models(context, models, exclude):
    """
    Returns (model, perm,) for all models that match models/exclude patterns
    and are visible by current user.
    """
    items = get_avail_models(context)
    included = []
    full_name = lambda m: '%s.%s' % (m.__module__, m.__name__)

    # I believe that that implemented
    # O(len(patterns)*len(matched_patterns)*len(all_models))
    # algorithm is fine for model lists because they are small and admin
    # performance is not a bottleneck. If it is not the case then the code
    # should be optimized.

    if len(models) == 0:
        included = items
    else:
        for pattern in models:
            for item in items:
                model, perms = item
                if fnmatch(full_name(model), pattern) and item not in included:
                    included.append(item)

    result = included[:]
    for pattern in exclude:
        for item in included:
            model, perms = item
            if fnmatch(full_name(model), pattern):
                try:
                    result.remove(item)
                except ValueError:  # if the item was already removed skip
                    pass
    return result


class UserTestElementMixin(object):
    """
    Mixin which adds a method for checking if current user is allowed to see
    something (menu, menu item, etc.).
    """

    def is_user_allowed(self, user):
        """
        This method can be overwritten to check if current user can see this
        element.
        """
        return True


class AppListElementMixin(object):
    """
    Mixin class for AppList and ModelList MenuItem.
    """

    def _visible_models(self, context):

        included = self.models[:]
        excluded = self.exclude[:]

        if excluded and not included:
            included = ["*"]
        return filter_models(context, included, excluded)

    def _get_admin_app_list_url(self, model, context):
        """
        Returns the admin change url.
        """
        app_label = model._meta.app_label
        return reverse('%s:app_list' % get_admin_site_name(context),
                       args=(app_label,))

    def _get_admin_change_url(self, model, context):
        """
        Returns the admin change url.
        """
        app_label = model._meta.app_label
        return reverse('%s:%s_%s_changelist' % (get_admin_site_name(context),
                                                app_label,
                                                model.__name__.lower()))

    def _get_admin_add_url(self, model, context):
        """
        Returns the admin add url.
        """
        app_label = model._meta.app_label
        return reverse('%s:%s_%s_add' % (get_admin_site_name(context),
                                         app_label,
                                         model.__name__.lower()))

    def is_empty(self):
        return len(self.children) == 0




from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings

from wpadmin.utils import get_admin_site_name
from wpadmin.menu.utils import UserTestElementMixin
from wpadmin.menu import items


class Menu(UserTestElementMixin):
    """
    Base menu.
    """
    children = None

    def __init__(self, **kwargs):
        for key in kwargs:
            if hasattr(self.__class__, key):
                setattr(self, key, kwargs[key])
        self.children = kwargs.get('children', [])

    def init_with_context(self, context):
        pass


class DefaultTopMenu(Menu):
    """
    Default top menu which mimics default Django admin header.
    """

    def init_with_context(self, context):

        self.children += [
            items.MenuItem(
                # Translators: This is already translated in Django
                title=_('Django administration'),
                url=None,
                icon='fa-gears',
                css_styles='font-size: 1.5em;',
            ),
            items.UserTools(
                css_styles='float: right;',
                check_if_user_allowed=lambda user: user.is_active and user.is_staff,
            ),
        ]


class BasicTopMenu(Menu):
    """
    Basic default top menu.
    """

    def init_with_context(self, context):

        admin_site_name = get_admin_site_name(context)

        if 'django.contrib.sites' in settings.INSTALLED_APPS:
            from django.contrib.sites.models import Site
            site_name = Site.objects.get_current().name
            site_url = 'http://' + Site.objects.get_current().domain
        else:
            site_name = _('Site')
            site_url = '/'

        self.children += [
            items.MenuItem(
                title=site_name,
                url=site_url,
                icon='fa-bullseye',
                css_styles='font-size: 1.5em;',
            ),
            items.MenuItem(
                title=_('Dashboard'),
                icon='fa-tachometer',
                url=reverse('%s:index' % admin_site_name),
                description=_('Dashboard'),
            ),
            items.UserTools(
                css_styles='float: right;',
                check_if_user_allowed=lambda user: user.is_staff,
            ),
        ]


class BasicLeftMenu(Menu):
    """
    Basic default left menu.
    """

    def is_user_allowed(self, user):
        """
        Only users that are staff are allowed to see this menu.
        """
        return user.is_staff

    def init_with_context(self, context):

        if self.is_user_allowed(context.get('request').user):

            admin_site_name = get_admin_site_name(context)

            self.children += [
                items.MenuItem(
                    title=_('Dashboard'),
                    icon='fa-tachometer',
                    url=reverse('%s:index' % admin_site_name),
                    description=_('Dashboard'),
                ),
                items.AppList(
                    title=_('Applications'),
                    description=_('Applications'),
                    exclude=('django.contrib.*',),
                    icon='fa-tasks',
                ),
                items.AppList(
                    title=_('Administration'),
                    description=_('Administration'),
                    models=('django.contrib.*',),
                    icon='fa-cog',
                ),
            ]




"""
django_wpadmin is a collection of extensions/tools for the default Django
administration panel which makes it look and behave more like WordPress
administration panel.
"""
VERSION = (1, 6, 0)

try:
    from django import VERSION as DJANGO_VERSION

    if VERSION[:2] != DJANGO_VERSION[:2]:
        import warnings
        warnings.warn(
            'Django WP Admin %(wpadmin_version)s is not compatible with '
            'Django %(django_version)s and may not work properly. '
            'You should install Django WP Admin from %(branch)s branch.'
            % {'wpadmin_version': '.'.join(str(x) for x in VERSION),
               'django_version': '.'.join(str(x) for x in DJANGO_VERSION[:3]),
               'branch': '.'.join([str(x) for x in DJANGO_VERSION[:2]] + ['x'])},
            UserWarning)
except ImportError:
    pass



from django import template
from django.utils.translation import ugettext_lazy as _

from wpadmin.utils import (
    get_admin_site_name, get_wpadmin_settings, are_breadcrumbs_enabled)

register = template.Library()


def wpadmin_render_custom_style(context):
    custom_style_path = get_wpadmin_settings(get_admin_site_name(context)) \
        .get('custom_style', None)
    if custom_style_path:
        return '<link type="text/css" rel="stylesheet" href="%s" />' \
            % custom_style_path
    else:
        return ''

register.simple_tag(takes_context=True)(wpadmin_render_custom_style)


class AreBreadcrumbsEnabledNode(template.Node):

    def render(self, context):
        context['wpadmin_are_breadcrumbs_enabled'] = are_breadcrumbs_enabled(
            get_admin_site_name(context))
        return ''


def wpadmin_are_breadcrumbs_enabled(parser, token):
    return AreBreadcrumbsEnabledNode()

register.tag('wpadmin_are_breadcrumbs_enabled', wpadmin_are_breadcrumbs_enabled)


def wpadmin_render_custom_title(context):
    # Translators: This is already translated in Django
    return get_wpadmin_settings(get_admin_site_name(context)) \
        .get('title', _('Django site admin'))

register.simple_tag(takes_context=True)(wpadmin_render_custom_title)




"""
WPadmin utilities.
"""

from django.conf import settings
from django.utils.importlib import import_module
from django.contrib import admin
from django.utils.translation import get_language_from_path


def get_wpadmin_settings(admin_site_name='admin'):
    """
    Get WPadmin settings for specified admin site.
    """
    return getattr(settings, 'WPADMIN', {}).get(admin_site_name, {})


def get_admin_site_name(context):
    """
    Get admin site name from request from context.
    Admin site name is taken from request path:
    * it is first part of path - between first and second slash if there is no
    lang prefix
    * or second part fo path - between second and third slash
    """
    path = context.get('request').path
    lang = get_language_from_path(path)
    path = path.split('/')
    if lang and path[1] == lang:
        return path[2]
    return path[1]


def get_admin_site(context):
    """
    Get admin site instance.
    """
    admin_site = get_wpadmin_settings(get_admin_site_name(context)) \
        .get('admin_site')
    if admin_site:
        mod, inst = admin_site.rsplit('.', 1)
        mod = import_module(mod)
        return getattr(mod, inst)
    else:
        return admin.site


def are_breadcrumbs_enabled(admin_site_name='admin'):
    """
    """
    return get_wpadmin_settings(admin_site_name).get('dashboard', {}) \
        .get('breadcrumbs', True)




import re

from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _

from wpadmin.menu.utils import UserTestElementMixin, AppListElementMixin


class MenuItem(UserTestElementMixin):
    """
    This is the base class for custom menu items.
    A menu item can have the following properties:

    ``title``
        String that contains the menu item title, make sure you use the
        django gettext functions if your application is multilingual.
        Default value: 'Untitled menu item'.

    ``url``
        String that contains the menu item URL.
        Default value: None (will be rendered as 'javascript:;').

    ``add_url``
        An optional string that contains second menu item URL. This url allows
        to have edit and add urls in one menu item. add_url is small plus sign
        in menu, next to normal url.
        Default value: None.

    ``icon``
        An optional string which contains classes for icons from Font Awesome
        which should be used for this menu item. Note that icons may not show
        on all levels of menu. They are only supported at top level.
        Default value: None.

    ``css_styles``
        String containing special CSS styling for this menu item.
        Default value: None.

    ``description``
        An optional string that will be used as the ``title`` attribute of
        the menu-item ``a`` tag.
        Default value: None.

    ``enabled``
        Boolean that determines whether the menu item is enabled or not.
        Disabled items are displayed but are not clickable.
        Default value: True.

    ``children``
        A list of children menu items. All children items must be instances of
        the ``MenuItem`` class.
    """

    title = 'Untitled menu item'
    url = None
    add_url = None
    icon = None
    css_styles = None
    description = None
    enabled = True
    children = None

    def __init__(self, title=None, url=None, **kwargs):

        if title is not None:
            self.title = title

        if url is not None:
            self.url = url

        for key in kwargs:
            if hasattr(self.__class__, key):
                setattr(self, key, kwargs[key])
        self.children = self.children or []

    def init_with_context(self, context):
        pass

    def is_selected(self, request):
        """
        Helper method that returns ``True`` if the menu item is active.
        A menu item is considered as active if it's url or add_url or one of its
        descendants url or add_url is equals to the current URL.
        """
        current_url = request.path

        return self.url == current_url or \
            self.add_url == current_url or \
            bool(re.match('^%s\d+/$' % self.url, current_url)) or \
            len([c for c in self.children if c.is_selected(request)]) > 0

    def is_empty(self):
        """
        Helper method that returns ``True`` if the menu item is empty.
        This method always returns ``False`` for basic items, but can return
        ``True`` if the item is an AppList.
        """
        return False


class AppList(AppListElementMixin, MenuItem):
    """
    """

    def __init__(self, title=None, models=None, exclude=None, **kwargs):
        self.models = list(models or [])
        self.exclude = list(exclude or [])
        super(AppList, self).__init__(title, **kwargs)

    def init_with_context(self, context):

        items = self._visible_models(context)
        apps = {}
        for model, perms in items:
            if not perms['change'] and not perms['add']:
                continue
            app_label = model._meta.app_label
            if app_label not in apps:
                apps[app_label] = {
                    'title': capfirst(_(app_label.title())),
                    'url': self._get_admin_app_list_url(model, context),
                    'models': []
                }
            apps[app_label]['models'].append({
                'title': capfirst(model._meta.verbose_name_plural),
                'url': perms['change'] and self._get_admin_change_url(
                    model, context),
                'add_url': perms['add'] and self._get_admin_add_url(
                    model, context),
                'description':
                # Translators: This is already translated in Django
                perms['change'] and _("Change") 
                # Translators: This is already translated in Django
                or _("No permission"),
            })

        apps_sorted = list(apps.keys())
        apps_sorted.sort()
        for app in sorted(apps.keys()):
            app_dict = apps[app]
            item = MenuItem(
                title=app_dict['title'], url=app_dict['url'],
                description=app_dict['title'])
            # sort model list alphabetically
            apps[app]['models'].sort(key=lambda x: x['title'])
            for model_dict in apps[app]['models']:
                item.children.append(MenuItem(**model_dict))
            self.children.append(item)


class ModelList(AppListElementMixin, MenuItem):
    """
    """

    def __init__(self, title=None, models=None, exclude=None, **kwargs):
        self.models = list(models or [])
        self.exclude = list(exclude or [])

        super(ModelList, self).__init__(title, **kwargs)

    def init_with_context(self, context):

        items = self._visible_models(context)
        for model, perms in items:
            if not perms['change']:
                continue
            title = capfirst(model._meta.verbose_name_plural)
            url = self._get_admin_change_url(model, context)
            add_url = self._get_admin_add_url(model, context)
            item = MenuItem(
                title=title, url=url, description=title, add_url=add_url)
            self.children.append(item)


class UserTools(MenuItem):
    """
    """
    is_user_tools = True




import os
from setuptools import setup, find_packages
from wpadmin import VERSION


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='django-wpadmin',
    version='.'.join(str(x) for x in VERSION),
    description='WordPress look and feel for Django administration panel.',
    long_description = '''
%s

%s
''' % (read('README.rst'), read('docs/changelog.rst')),
    author='Maciej Marczewski (barszcz)',
    author_email='maciej@marczewski.net.pl',
    url = 'https://github.com/barszczmm/django-wpadmin',
    keywords = 'wordpress django admin',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development',
        'Topic :: Software Development :: User Interfaces',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    zip_safe = False,
)


import urllib
import hashlib
from django import template

from wpadmin.utils import get_admin_site_name
from wpadmin.menu.utils import get_menu

register = template.Library()


class IsMenuEnabledNode(template.Node):

    def __init__(self, menu_name):
        """
        menu_name - menu name ('top' or 'left')
        """
        self.menu_name = menu_name

    def render(self, context):
        menu = get_menu(self.menu_name, get_admin_site_name(context))
        if menu and menu.is_user_allowed(context.get('request').user):
            enabled = True
        else:
            enabled = False
        context['wpadmin_is_%s_menu_enabled' % self.menu_name] = enabled
        return ''


def wpadmin_is_left_menu_enabled(parser, token):
    return IsMenuEnabledNode('left')

register.tag('wpadmin_is_left_menu_enabled', wpadmin_is_left_menu_enabled)


def wpadmin_render_top_menu(context):
    menu = get_menu('top', get_admin_site_name(context))
    if not menu:
        from wpadmin.menu.menus import DefaultTopMenu
        menu = DefaultTopMenu()
    menu.init_with_context(context)
    context.update({
        'menu': menu,
        'is_user_allowed': menu.is_user_allowed(context.get('request').user),
    })
    return context

register.inclusion_tag(
    'wpadmin/menu/top_menu.html',
    takes_context=True)(wpadmin_render_top_menu)


def wpadmin_render_left_menu(context):
    menu = get_menu('left', get_admin_site_name(context))
    if menu:
        menu.init_with_context(context)
        context.update({
            'menu': menu,
            'is_user_allowed': menu.is_user_allowed(context.get('request').user),
        })
    return context

register.inclusion_tag(
    'wpadmin/menu/left_menu.html',
    takes_context=True)(wpadmin_render_left_menu)


def wpadmin_render_menu_top_item(context, item, is_first, is_last):
    item.init_with_context(context)
    if item.icon:
        icon = item.icon
    else:
        icon = 'fa-folder-o'
    context.update({
        'item': item,
        'is_first': is_first,
        'is_last': is_last,
        'icon': icon,
        'is_selected': item.is_selected(context.get('request')),
        'is_user_allowed': item.is_user_allowed(context.get('request').user),
    })
    return context

register.inclusion_tag(
    'wpadmin/menu/menu_top_item.html',
    takes_context=True)(wpadmin_render_menu_top_item)


def wpadmin_render_menu_item(context, item, is_first, is_last):
    item.init_with_context(context)
    context.update({
        'item': item,
        'is_first': is_first,
        'is_last': is_last,
        'is_selected': item.is_selected(context.get('request')),
        'is_user_allowed': item.is_user_allowed(context.get('request').user),
    })
    return context

register.inclusion_tag(
    'wpadmin/menu/menu_item.html',
    takes_context=True)(wpadmin_render_menu_item)


def wpadmin_render_user_tools(context, item, is_first, is_last):
    item.init_with_context(context)
    context.update({
        'item': item,
        'is_first': is_first,
        'is_last': is_last,
        'is_user_allowed': item.is_user_allowed(context.get('request').user),
    })
    return context

register.inclusion_tag(
    'wpadmin/menu/user_tools.html',
    takes_context=True)(wpadmin_render_user_tools)


def gravatar_url(user, size, https=True):
    default = 'wavatar'
    if https:
        url = 'https'
    else:
        url = 'http'
    url += '://www.gravatar.com/avatar.php?'
    url += urllib.urlencode({
        'gravatar_id': hashlib.md5(user.email.lower()).hexdigest(),
        'default': default,
        'size': str(size)})
    return url

register.simple_tag(gravatar_url)

